Crate ssh_cipher

Crate ssh_cipher 

RustCrypto: SSH Symmetric Ciphers

crate Docs Build Status Apache2/MIT licensed Rust Version Project Chat

Documentation

About

Pure Rust implementation of SSH symmetric encryption including support for the modern aes128-gcm@openssh.com/aes256-gcm@openssh.com and chacha20-poly1305@openssh.com algorithms as well as legacy support for older ciphers.

Built on the pure Rust cryptography implementations maintained by the RustCrypto organization.

License

Licensed under either of:

at your option.

Contribution

Unless you explicitly state otherwise, any contribution intentionally submitted for inclusion in the work by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any additional terms or conditions.

Re-exports

pub use cipher;

Modules

block_cipheraes or tdes
Low-level block cipher interface.

Structs

ChaCha20Poly1305chacha20poly1305
OpenSSH variant of ChaCha20Poly1305: chacha20-poly1305@openssh.com as described in PROTOCOL.chacha20poly1305.

Enums

Cipher
Cipher algorithms.
Error
Error type.

Type Aliases

AesGcmNonceaes
Nonce for aes128-gcm@openssh.com/aes256-gcm@openssh.com.
ChaCha20chacha20poly1305
The ChaCha20 stream cipher (legacy “djb” construction with 64-bit nonce).
ChaChaKeychacha20poly1305
Key for chacha20-poly1305@openssh.com.
ChaChaNoncechacha20poly1305
Nonce for chacha20-poly1305@openssh.com.
Result
Result type with ssh-cipher crate’s Error as the error type.
Tag
Authentication tag for ciphertext data.