Module block_cipher

Module block_cipher 

Available on crate features aes or tdes only.
Low-level block cipher interface.

This module provides APIs which enable streaming and “peeking” when using unauthenticated block cipher modes such as CBC and CTR.

Structs

Aesaes
Advanced Encryption Standard (AES) low-level block cipher.
Decryptor
Stateful decryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.
Encryptor
Stateful encryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.
InvalidLength
The error type returned when key and/or IV used in the KeyInit, KeyIvInit, and InnerIvInit slice-based methods had an invalid length.
Tdestdes
Triple DES (3DES) block cipher.

Traits

BlockModeDecrypt
Decrypt-only functionality for block ciphers and modes with mutable access to self.
BlockModeEncrypt
Encrypt-only functionality for block ciphers and modes with mutable access to self.
BlockSizeUser
Types which process data in blocks.

Type Aliases

AesDecryptoraes
Decryptor for the Advanced Encryption Standard (AES).
AesEncryptoraes
Encryptor for the Advanced Encryption Standard (AES).
Block
Block on which BlockSizeUser implementors operate.
TdesDecryptortdes
Decryptor for 3DES.
TdesEncryptortdes
Encryptor for 3DES.