Decryptor

Struct Decryptor 

pub struct Decryptor<C: BlockCipher> { /* private fields */ }
Available on crate features aes or tdes only.
Stateful decryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.

Note we need encryption support for decryption in order to support AES-CTR, where encryption and decryption are the same operation.

Implementations

Source

impl<C: BlockCipher> Decryptor<C>

pub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>

Create a new decryptor object with the given Cipher, key, and iv (i.e. initialization vector).

Errors
  • Returns Error::Crypto if the given cipher cannot be used with Decryptor.
  • Returns Error::Length if key or iv are the wrong length for the given cipher.
  • Returns Error::UnsupportedCipher if support for the given cipher is not enabled in the crate features.
pub fn peek<T, F>(&mut self, f: F) -> T
where F: FnMut(&mut Self) -> T,

Call the provided function with an ephemeral Decryptor state which will be reset upon completion, returning the result of the function.