pub struct Encryptor<C: BlockCipher> { /* private fields */ }Available on crate features
aes or tdes only.Expand description
Stateful encryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.
Implementations
Sourceimpl<C: BlockCipher> Encryptor<C>
impl<C: BlockCipher> Encryptor<C>
Sourcepub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>
pub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>
Create a new encryptor object with the given Cipher, key, and iv (i.e.
initialization vector).
Errors
- Returns
Error::Cryptoif the givenciphercannot be used withEncryptor. - Returns
Error::Lengthifkeyorivare the wrong length for the givencipher. - Returns
Error::UnsupportedCipherif support for the givencipheris not enabled in the crate features.