pub struct Decryptor { /* private fields */ }Available on crate features
aes-cbc or aes-ctr or tdes only.Expand description
Stateful decryptor object for unauthenticated SSH symmetric ciphers.
Note that this deliberately does not support AEAD modes such as AES-GCM and ChaCha20Poly1305, which are one-shot by design.
Implementations
Sourceimpl Decryptor
impl Decryptor
Sourcepub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>
pub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>
Create a new decryptor object with the given Cipher, key, and iv (i.e.
initialization vector).
Errors
- Returns
Error::Lengthifkeyorivare the wrong length for the givencipher. - Returns
Error::UnsupportedCipherif support for the givencipheris not enabled in the crate features.