RustCrypto: SSH Symmetric Ciphers
About
Pure Rust implementation of SSH symmetric encryption including support for the
modern aes128-gcm@openssh.com/aes256-gcm@openssh.com and
chacha20-poly1305@openssh.com algorithms as well as legacy support for older
ciphers.
Built on the pure Rust cryptography implementations maintained by the RustCrypto organization.
License
Licensed under either of:
at your option.
Contribution
Unless you explicitly state otherwise, any contribution intentionally submitted for inclusion in the work by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any additional terms or conditions.
Re-exports
pub use cipher;
Modules
- block_
cipher aesortdes - Low-level block cipher interface.
Structs
- ChaCha20
Poly1305 chacha20poly1305 - OpenSSH variant of ChaCha20Poly1305:
chacha20-poly1305@openssh.comas described in PROTOCOL.chacha20poly1305.
Enums
Type Aliases
- AesGcm
Nonce aes - Nonce for
aes128-gcm@openssh.com/aes256-gcm@openssh.com. - ChaCha20
chacha20poly1305 - The ChaCha20 stream cipher (legacy “djb” construction with 64-bit nonce).
- ChaCha
Key chacha20poly1305 - Key for
chacha20-poly1305@openssh.com. - ChaCha
Nonce chacha20poly1305 - Nonce for
chacha20-poly1305@openssh.com. - Result
- Result type with
ssh-ciphercrate’sErroras the error type. - Tag
- Authentication tag for ciphertext data.