Available on crate features
aes or tdes only.Expand description
Low-level block cipher interface.
This module provides APIs which enable streaming and “peeking” when using unauthenticated block cipher modes such as CBC and CTR.
Structs
- Aes
aes - Advanced Encryption Standard (AES) low-level block cipher.
- Decryptor
- Stateful decryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.
- Encryptor
- Stateful encryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.
- Invalid
Length - The error type returned when key and/or IV used in the
KeyInit,KeyIvInit, andInnerIvInitslice-based methods had an invalid length. - Tdes
tdes - Triple DES (3DES) block cipher.
Traits
- Block
Mode Decrypt - Decrypt-only functionality for block ciphers and modes with mutable access to
self. - Block
Mode Encrypt - Encrypt-only functionality for block ciphers and modes with mutable access to
self. - Block
Size User - Types which process data in blocks.
Type Aliases
- AesDecryptor
aes - Decryptor for the Advanced Encryption Standard (AES).
- AesEncryptor
aes - Encryptor for the Advanced Encryption Standard (AES).
- Block
- Block on which
BlockSizeUserimplementors operate. - Tdes
Decryptor tdes - Decryptor for 3DES.
- Tdes
Encryptor tdes - Encryptor for 3DES.