pub struct Decryptor<C: BlockCipher> { /* private fields */ }Available on crate features
aes or tdes only.Expand description
Stateful decryptor object for unauthenticated symmetric ciphers used in the SSH packet encryption protocol.
Note we need encryption support for decryption in order to support AES-CTR, where encryption and decryption are the same operation.
Implementations
Sourceimpl<C: BlockCipher> Decryptor<C>
impl<C: BlockCipher> Decryptor<C>
Sourcepub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>
pub fn new(cipher: Cipher, key: &[u8], iv: &[u8]) -> Result<Self>
Create a new decryptor object with the given Cipher, key, and iv (i.e.
initialization vector).
Errors
- Returns
Error::Cryptoif the givenciphercannot be used withDecryptor. - Returns
Error::Lengthifkeyorivare the wrong length for the givencipher. - Returns
Error::UnsupportedCipherif support for the givencipheris not enabled in the crate features.